@@ -4,7 +4,7 @@
  * GChemPaint library
  * bond.cc 
  *
- * Copyright (C) 2001-2008 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2001-2010 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -108,7 +108,7 @@ bool Bond::GetLine2DCoords (unsigned Num, double* x1, double* y1, double* x2, do
 		dy *= (BondDist / l);
 		// now, exclude symbols rectangles from the drawing
 		double ax, ay, anga, angb = atan2 (fabs (dy), fabs (dx));
-		reinterpret_cast <Atom *> (m_Begin)->GetSymbolGeometry (ax, ay, anga, dy < 0);
+		static_cast <Atom *> (m_Begin)->GetSymbolGeometry (ax, ay, anga, dy < 0);
 		bool horizontal;
 		if (ax > 0) {
 			horizontal = anga >= angb;
@@ -126,17 +126,17 @@ bool Bond::GetLine2DCoords (unsigned Num, double* x1, double* y1, double* x2, do
 				*x1 -= ay * dx / dy;
 			}
 		}
-		reinterpret_cast <Atom *> (m_End)->GetSymbolGeometry (ax, ay, anga, dy > 0);
+		static_cast <Atom *> (m_End)->GetSymbolGeometry (ax, ay, anga, dy > 0);
 		if (ax > 0) {
 			horizontal = anga >= angb;
 			if (horizontal) {
-				ax = (ax + 2.) / Theme->GetZoomFactor ();
+				ax = (ax + 1.) / Theme->GetZoomFactor ();
 				if (dx > 0)
 					ax = - ax;
 				*x2 += ax;
 				*y2 += ax * dy / dx;
 			} else {
-				ay = (ay + 2.) / Theme->GetZoomFactor ();
+				ay = (ay + 1.) / Theme->GetZoomFactor ();
 				if (dy > 0)
 					ay = - ay;
 				*y2 += ay;
@@ -192,11 +192,11 @@ bool Bond::GetLine2DCoords (unsigned Num, double* x1, double* y1, double* x2, do
 			if (sin(a0 - a1) * sin (a0 - a2) > 0) {
 				double sign = sin (a0 - a1) > 0.0 ? 1.0 : -1.0;
 				double tanb = 0., cosa = cos (a0), sina = sin (a0);
-				if (m_Begin->GetZ () == 6 && !reinterpret_cast <Atom*> (m_Begin)->GetShowSymbol ())
+				if (m_Begin->GetZ () == 6 && !static_cast <Atom*> (m_Begin)->GetShowSymbol ())
 					tanb = fabs (tan ((M_PI - a0 + a1) / 2));
 				m_coords[4] = *x1 + BondDist * cosa * tanb - dy * sign;
 				m_coords[5] = *y1 + dx * sign - BondDist * sina * tanb;
-				tanb = (m_End->GetZ () == 6 && !reinterpret_cast <Atom*> (m_End)->GetShowSymbol ())? fabs (tan ((a2 - a0) / 2)): 0.;
+				tanb = (m_End->GetZ () == 6 && !static_cast <Atom*> (m_End)->GetShowSymbol ())? fabs (tan ((a2 - a0) / 2)): 0.;
 				m_coords[6] = *x2 - BondDist * cosa * tanb - dy * sign;
 				m_coords[7] = *y2 + dx * sign + BondDist * sina * tanb;
 			} else goto general;
@@ -207,60 +207,57 @@ general:
 			if (n1 == 1) {
 				// put the second line on the bond side if any
 				map <gcu::Atom*, gcu::Bond*>::iterator it;
-				Bond *bond = reinterpret_cast <Bond*> (m_Begin->GetFirstBond (it));
+				Bond *bond = static_cast <Bond*> (m_Begin->GetFirstBond (it));
 				if (bond == this)
-					bond = reinterpret_cast <Bond*> (m_Begin->GetNextBond (it));
-				double a0 = atan2 (*y1 - *y2, *x2 - *x1), a1 = bond->GetAngle2DRad (reinterpret_cast <Atom*> (m_Begin)), a2, a;
+					bond = static_cast <Bond*> (m_Begin->GetNextBond (it));
+				double a0 = atan2 (*y1 - *y2, *x2 - *x1), a1 = bond->GetAngle2DRad (static_cast <Atom*> (m_Begin)), a;
 				if (fabs (fabs (a0 - a1) - M_PI) > 0.01) {
 					double sign = sin (a0 - a1) > 0.0 ? 1.0 : -1.0;
 					double tanb = ((m_Begin->GetZ () == 6)? fabs (tan ((M_PI - a0 + a1) / 2)): 0.), cosa = cos (a0), sina = sin (a0);
 					m_coords[4] = *x1 + BondDist * cosa * tanb - dy * sign;
 					m_coords[5] = *y1 + dx * sign - BondDist * sina * tanb;
 					tanb = 0.;
-					a2 = M_PI + a0;
-					if (a2 > 2 * M_PI)
-						a2 -= 2 * M_PI;
-					bond = reinterpret_cast <Bond*> (m_End->GetFirstBond (it));
+					bond = static_cast <Bond*> (m_End->GetFirstBond (it));
 					if (m_End->GetZ () == 6)
 						while (bond) {
 							if (bond != this) {
-								a = tan ((bond->GetAngle2DRad (reinterpret_cast <Atom*> (m_End)) - a0) / 2);
+								a = tan ((bond->GetAngle2DRad (static_cast <Atom*> (m_End)) - a0) / 2);
 								if (sign * a < sign * tanb)
 									tanb = a;
 								
 							}
-							bond = reinterpret_cast <Bond*> (m_End->GetNextBond (it));
+							bond = static_cast <Bond*> (m_End->GetNextBond (it));
 						}
-					m_coords[6] = *x2 + BondDist * cosa * tanb - dy * sign;
-					m_coords[7] = *y2 + dx * sign - BondDist * sina * tanb;
+					m_coords[6] = *x2 + (BondDist * cosa * tanb - dy) * sign;
+					m_coords[7] = *y2 + (dx - BondDist * sina * tanb) * sign;
 					goto done;
 				}
 			} else if (n1 > 1 && n2 > 0) {
 				if (n2 == 1) {
 					map <gcu::Atom*, gcu::Bond*>::iterator it;
-					Bond *bond = reinterpret_cast <Bond*> (m_End->GetFirstBond (it));
+					Bond *bond = static_cast <Bond*> (m_End->GetFirstBond (it));
 					if (bond == this)
-						bond = reinterpret_cast <Bond*> (m_End->GetNextBond (it));
-					double a0 = atan2 (*y1 - *y2, *x2 - *x1), a1 = bond->GetAngle2DRad (reinterpret_cast <Atom*> (m_End)), a2, a;
+						bond = static_cast <Bond*> (m_End->GetNextBond (it));
+					double a0 = atan2 (*y1 - *y2, *x2 - *x1), a1 = bond->GetAngle2DRad (static_cast <Atom*> (m_End)), a2, a;
 					if (fabs (fabs (a0 - a1) - M_PI) > 0.01) {
 						double sign = sin (a0 - a1) > 0.0 ? 1.0 : -1.0;
 						double tanb = 0., cosa = cos (a0), sina = sin (a0);
 						a2 = M_PI + a0;
 						if (a2 > 2 * M_PI)
 							a2 -= 2 * M_PI;
-						bond = reinterpret_cast <Bond*> (m_Begin->GetFirstBond (it));
+						bond = static_cast <Bond*> (m_Begin->GetFirstBond (it));
 						if (m_Begin->GetZ () == 6)
 							while (bond) {
 								if (bond != this) {
-									a = tan ((bond->GetAngle2DRad (reinterpret_cast <Atom*> (m_Begin)) - a2) / 2);
+									a = tan ((bond->GetAngle2DRad (static_cast <Atom*> (m_Begin)) - a2) / 2);
 									if (sign * a < sign * tanb)
 										tanb = a;
 									
 								}
-								bond = reinterpret_cast <Bond*> (m_Begin->GetNextBond (it));
+								bond = static_cast <Bond*> (m_Begin->GetNextBond (it));
 							}
-						m_coords[4] = *x1 - BondDist * cosa * tanb - dy * sign;
-						m_coords[5] = *y1 + dx * sign + BondDist * sina * tanb;
+						m_coords[4] = *x1 - (BondDist * cosa * tanb + dy) * sign;
+						m_coords[5] = *y1 + (dx + BondDist * sina * tanb) * sign;
 						tanb = (m_End->GetZ () == 6)? fabs (tan ((a1 - a0) / 2)): 0.;
 						m_coords[6] = *x2 - BondDist * cosa * tanb - dy * sign;
 						m_coords[7] = *y2 + dx * sign + BondDist * sina * tanb;
@@ -268,36 +265,36 @@ general:
 				} else {
 					// use the side with more room.
 					double tana = 0., tanb = 0., tanc = 0., tand = 0.;
-					double a, a0 = GetAngle2DRad (reinterpret_cast <Atom*> (m_End)), a1 = GetAngle2DRad (reinterpret_cast <Atom*> (m_Begin));
+					double a, a0 = GetAngle2DRad (static_cast <Atom*> (m_End)), a1 = GetAngle2DRad (static_cast <Atom*> (m_Begin));
 					double cosa = cos (a0), sina = sin (a0);
 					map <gcu::Atom*, gcu::Bond*>::iterator it;
 					Bond *bond;
 					if (m_Begin->GetZ () == 6) {
-						bond = reinterpret_cast <Bond*> (m_Begin->GetFirstBond (it));
+						bond = static_cast <Bond*> (m_Begin->GetFirstBond (it));
 						while (bond) {
 							if (bond != this) {
-								a = tan ((bond->GetAngle2DRad (reinterpret_cast <Atom*> (m_Begin)) - a0) / 2);
+								a = tan ((bond->GetAngle2DRad (static_cast <Atom*> (m_Begin)) - a0) / 2);
 								if (a > tana)
 									tana = a;
 								if (a < tanb)
 									tanb = a;
 								
 							}
-							bond = reinterpret_cast <Bond*> (m_Begin->GetNextBond (it));
+							bond = static_cast <Bond*> (m_Begin->GetNextBond (it));
 						}
 					}
 					if (m_End->GetZ () == 6) {
-						bond = reinterpret_cast <Bond*> (m_End->GetFirstBond (it));
+						bond = static_cast <Bond*> (m_End->GetFirstBond (it));
 						while (bond) {
 							if (bond != this) {
-								a = tan ((bond->GetAngle2DRad (reinterpret_cast <Atom*> (m_End)) - a1) / 2);
+								a = tan ((bond->GetAngle2DRad (static_cast <Atom*> (m_End)) - a1) / 2);
 								if (a > tanc)
 									tanc = a;
 								if (a < tand)
 									tand = a;
 								
 							}
-							bond = reinterpret_cast <Bond*> (m_End->GetNextBond (it));
+							bond = static_cast <Bond*> (m_End->GetNextBond (it));
 						}
 					}
 					if (tana - tand > tanc - tanb) {
@@ -315,10 +312,10 @@ general:
 				goto done;
 			} else if (n2 == 1) { // n1 is 0
 				map <gcu::Atom*, gcu::Bond*>::iterator it;
-				Bond *bond = reinterpret_cast <Bond*> (m_End->GetFirstBond (it));
+				Bond *bond = static_cast <Bond*> (m_End->GetFirstBond (it));
 				if (bond == this)
-					bond = reinterpret_cast <Bond*> (m_End->GetNextBond (it));
-				double a0 = atan2 (*y1 - *y2, *x2 - *x1), a1 = bond->GetAngle2DRad (reinterpret_cast <Atom*> (m_End));
+					bond = static_cast <Bond*> (m_End->GetNextBond (it));
+				double a0 = atan2 (*y1 - *y2, *x2 - *x1), a1 = bond->GetAngle2DRad (static_cast <Atom*> (m_End));
 				if (fabs (fabs (a0 - a1) - M_PI) > 0.01) {
 					double sign = sin (a0 - a1) > 0.0 ? 1.0 : -1.0;
 					double tanb, cosa = cos (a0), sina = sin (a0);
@@ -607,7 +604,7 @@ void Bond::AddItem ()
 				line->SetLineColor (0);
 				line->SetOperator (CAIRO_OPERATOR_SOURCE);
 				/* FIXME: unsecure if several bonds are crossing the same bond, use masking instead */
-				view->GetCanvas ()->GetRoot ()->MoveToBack (group);
+				view->GetCanvas ()->GetRoot ()->MoveToFront (group);
 			}
 		}
 		break;
@@ -655,7 +652,15 @@ void Bond::AddItem ()
 	}
 	case DownBondType: {
 		GetLine2DCoords (1, &x1, &y1, &x2, &y2);
-		gccv::Hash *hash = new gccv::Hash (view->GetCanvas ()->GetRoot (),
+		gccv::Hash *hash = InvertWedgeHashes?
+			new gccv::Hash (view->GetCanvas ()->GetRoot (),
+							x1 * theme->GetZoomFactor (),
+							y1 * theme->GetZoomFactor (),
+							x2 * theme->GetZoomFactor (),
+							y2 * theme->GetZoomFactor (),
+							theme->GetStereoBondWidth (),
+							this):
+			new gccv::Hash (view->GetCanvas ()->GetRoot (),
 							x2 * theme->GetZoomFactor (),
 							y2 * theme->GetZoomFactor (),
 							x1 * theme->GetZoomFactor (),
@@ -679,358 +684,6 @@ void Bond::UpdateItem ()
 	}
 	AddItem ();
 }
-
-/*void Bond::Add (GtkWidget* w) const
-{
-	if (!w)
-		return;
-	WidgetData* pData = (WidgetData*) g_object_get_data (G_OBJECT (w), "data");
-	if (pData->Items[this] != NULL)
-		return;
-	Theme *pTheme = pData->m_View->GetDoc ()->GetTheme ();
-	gcu::Atom *pAtom0, *pAtom1;
-	if (!(pAtom0 = GetAtom (0)))
-		return;
-	if (!(pAtom1 = GetAtom (1)))
-		return;
-	unsigned char order = GetOrder ();
-	if (order == 0)
-		return;
-	GnomeCanvasGroup* group;
-	GnomeCanvasItem* item = NULL;
-	group = GNOME_CANVAS_GROUP (gnome_canvas_item_new (pData->Group, gnome_canvas_group_ext_get_type (), NULL));
-	g_signal_connect (G_OBJECT (group), "event", G_CALLBACK (on_event), w);
-	g_object_set_data (G_OBJECT (group), "object", (void *) this);
-	bool result = false;
-	if (m_Crossing.size () > 0) {
-		map<Bond*, BondCrossing>::const_iterator i, iend = m_Crossing.end ();
-		for (i = m_Crossing.begin (); i != iend; i++)
-			if ((result |= (*i).second.is_before))
-				break;
-	}
-	GnomeCanvasPathDef *path = NULL;
-	if (result) {
-		path = const_cast <Bond *> (this)->BuildCrossingPathDef (pData);
-		if (path) {
-			GnomeCanvasItem *item = (m_type == NormalBondType || m_type == UndeterminedBondType)?
-				gnome_canvas_item_new (
-					group,
-					gnome_canvas_bpath_ext_get_type (),
-					"bpath", path,
-					"outline_color", "white",
-					"width_units", pTheme->GetBondWidth () * 3.,
-					NULL):
-				gnome_canvas_item_new (
-					group,
-					gnome_canvas_bpath_ext_get_type (),
-					"bpath", path,
-					"fill_color", "white",
-					"width_units", 0.,
-					NULL);
-			g_object_set_data (G_OBJECT (group), "back", item);
-			g_object_set_data (G_OBJECT (item), "object", (void *) this);
-			g_signal_connect (G_OBJECT (item), "event", G_CALLBACK (on_event), w);*/
-			/* now bring to front, FIXME: not secure ! */
-/*			gnome_canvas_item_lower_to_bottom (item);
-			gnome_canvas_item_raise_to_top (GNOME_CANVAS_ITEM (group));
-			gcu::Atom *pAtom = GetAtom (0);
-			if (pAtom->GetZ () != 6 || static_cast <gcp::Atom*> (pAtom)->GetShowSymbol ())
-				gnome_canvas_item_raise_to_top (GNOME_CANVAS_ITEM (pData->Items[pAtom]));
-			pAtom = GetAtom (1);
-			if (pAtom->GetZ () != 6 || static_cast <gcp::Atom*> (pAtom)->GetShowSymbol ())
-				gnome_canvas_item_raise_to_top (GNOME_CANVAS_ITEM (pData->Items[pAtom]));
-			gnome_canvas_path_def_unref (path);
-		}
-	}
-	path = const_cast <Bond *> (this)->BuildPathDef (pData);
-	switch (GetType ())
-	{
-		case NormalBondType:
-		case UndeterminedBondType:
-			item = gnome_canvas_item_new(
-									group,
-									gnome_canvas_bpath_ext_get_type (),
-									"bpath", path,
-									"outline_color", (pData->IsSelected (this))? SelectColor: Color,
-									"width_units", pTheme->GetBondWidth (),
-									NULL);
-			break;
-		case ForeBondType:
-		case UpBondType:
-		case DownBondType:
-			item = gnome_canvas_item_new (
-									group,
-									gnome_canvas_bpath_ext_get_type (),
-									"bpath", path,
-									"fill_color", (pData->IsSelected (this))? SelectColor: Color,
-									"width_units", 0.,
-									NULL);
-			break;
-	}
-	gnome_canvas_path_def_unref (path);
-	g_object_set_data (G_OBJECT (group), "path", item);
-	g_object_set_data (G_OBJECT (item), "object", (void *) this);
-	g_signal_connect (G_OBJECT (item), "event", G_CALLBACK (on_event), w);
-	pData->Items[this] = group;
-	if (pAtom0->GetParent ()->GetType () == FragmentType)
-		gnome_canvas_item_raise_to_top (GNOME_CANVAS_ITEM (pData->Items[pAtom0->GetParent ()]));
-	else if (pAtom0->GetZ() != 6 || static_cast <Atom*> (pAtom0)->GetShowSymbol ())
-		gnome_canvas_item_raise_to_top (GNOME_CANVAS_ITEM (pData->Items[pAtom0]));
-	else
-		gnome_canvas_item_lower_to_bottom (GNOME_CANVAS_ITEM (pData->Items[pAtom0]));
-	map<string, Object*>::iterator i;
-	Object* electron = pAtom0->GetFirstChild (i);
-	while (electron) { 
-		gnome_canvas_item_raise_to_top (GNOME_CANVAS_ITEM (pData->Items[electron]));
-		electron = pAtom0->GetNextChild (i);
-	}
-	if (pAtom1->GetParent ()->GetType () == FragmentType)
-		gnome_canvas_item_raise_to_top (GNOME_CANVAS_ITEM (pData->Items[pAtom1->GetParent ()]));
-	else if (pAtom1->GetZ () != 6 || static_cast <Atom*> (pAtom1)->GetShowSymbol ())
-		gnome_canvas_item_raise_to_top (GNOME_CANVAS_ITEM (pData->Items[pAtom1]));
-	else
-		gnome_canvas_item_lower_to_bottom (GNOME_CANVAS_ITEM (pData->Items[pAtom1]));
-	electron = pAtom1->GetFirstChild (i);
-	while (electron) { 
-		gnome_canvas_item_raise_to_top (GNOME_CANVAS_ITEM (pData->Items[electron]));
-		electron = pAtom1->GetNextChild (i);
-	}
-}*/
-
-/*GnomeCanvasPathDef* Bond::BuildPathDef (WidgetData* pData)
-{
-	double x1, y1, x2, y2, x, y, dx, dy, dx1, dy1;
-	int i, n;
-	GnomeCanvasPathDef* path = gnome_canvas_path_def_new ();
-	Theme *pTheme = pData->m_View->GetDoc ()->GetTheme ();
-	switch (GetType ())
-	{
-		case NormalBondType:
-			i = 1;
-			while (GetLine2DCoords (i++, &x1, &y1, &x2, &y2))
-			{
-				gnome_canvas_path_def_moveto (path, x1 * pTheme->GetZoomFactor (), y1 * pTheme->GetZoomFactor ());
-				gnome_canvas_path_def_lineto (path, x2 * pTheme->GetZoomFactor (), y2 * pTheme->GetZoomFactor ());
-			}
-			break;
-		case UpBondType:
-			GetLine2DCoords(1, &x1, &y1, &x2, &y2);
-			gnome_canvas_path_def_moveto (path, x1 * pTheme->GetZoomFactor (), y1 * pTheme->GetZoomFactor ());
-			x = sqrt (square (x2 - x1) + square (y2 - y1));
-			dx = (y1 - y2) / x * pTheme->GetStereoBondWidth () / 2;
-			dy = (x2 - x1) / x * pTheme->GetStereoBondWidth () / 2;
-			gnome_canvas_path_def_lineto (path, x2 * pTheme->GetZoomFactor () + dx, y2 * pTheme->GetZoomFactor () + dy);
-			gnome_canvas_path_def_lineto (path, x2 * pTheme->GetZoomFactor () -dx, y2 * pTheme->GetZoomFactor () -dy);
-			gnome_canvas_path_def_closepath (path);
-			break;
-		case DownBondType:
-		{
-			double xi[8];
-			GetLine2DCoords (1, &x1, &y1, &x2, &y2);
-			x1 *= pTheme->GetZoomFactor ();
-			y1 *= pTheme->GetZoomFactor ();
-			x2 *= pTheme->GetZoomFactor ();
-			y2 *= pTheme->GetZoomFactor ();
-			x = sqrt (square (x2 - x1) + square (y2 - y1));
-			n = int (floor(x / (pTheme->GetHashDist () + pTheme->GetHashWidth ())));
-			dx1 = (x2 - x1) / x * pTheme->GetHashWidth ();
-			dy1 = (y2 - y1) / x * pTheme->GetHashWidth ();
-			dx = (y1 - y2) / x * pTheme->GetStereoBondWidth () / 2;
-			dy = (x2 - x1) / x * pTheme->GetStereoBondWidth () / 2;
-			gnome_canvas_path_def_moveto (path, xi[0] = x1 + dx, xi[1] = y1 + dy);
-			gnome_canvas_path_def_lineto (path, xi[2] = x1 - dx, xi[3] = y1 - dy);
-			dx *= (1 - pTheme->GetHashWidth () / x);
-			dy *= (1 - pTheme->GetHashWidth () / x);
-			gnome_canvas_path_def_lineto (path, xi[4] = x1 + dx1 - dx, xi[5] = y1 + dy1 - dy);
-			gnome_canvas_path_def_lineto (path, xi[6] = x1 + dx1 + dx, xi[7] = y1 + dy1 + dy);
-			gnome_canvas_path_def_lineto (path, xi[0], xi[1]);
-			gnome_canvas_path_def_closepath_current (path);
-			dx = (x2 - x1) / x * (pTheme->GetHashDist () + pTheme->GetHashWidth ())
-				- (y1 - y2) / x * pTheme->GetStereoBondWidth () / 2 * (pTheme->GetHashDist () + pTheme->GetHashWidth ()) / x;
-			dy = (y2 - y1) / x * (pTheme->GetHashDist () + pTheme->GetHashWidth ())
-				- (x2 - x1) / x * pTheme->GetStereoBondWidth () / 2 *  (pTheme->GetHashDist () + pTheme->GetHashWidth ()) / x;
-			dx1 = (x2 - x1) / x * (pTheme->GetHashDist () + pTheme->GetHashWidth ())
-				+ (y1 - y2) / x * pTheme->GetStereoBondWidth () / 2 *  (pTheme->GetHashDist () + pTheme->GetHashWidth ()) / x;
-			dy1 = (y2 - y1) / x * (pTheme->GetHashDist () + pTheme->GetHashWidth ())
-				+ (x2 - x1) / x * pTheme->GetStereoBondWidth () / 2 *  (pTheme->GetHashDist () + pTheme->GetHashWidth ()) / x;
-			for (int i = 1; i < n; i++) {
-				gnome_canvas_path_def_moveto (path, xi[0] += dx, xi[1] += dy);
-				gnome_canvas_path_def_lineto (path, xi[2] += dx1, xi[3] += dy1);
-				gnome_canvas_path_def_lineto (path, xi[4] += dx1, xi[5] += dy1);
-				gnome_canvas_path_def_lineto (path, xi[6] += dx, xi[7] += dy);
-				gnome_canvas_path_def_lineto (path, xi[0], xi[1]);
-				gnome_canvas_path_def_closepath_current (path);
-			}
-			break;
-		}
-		case ForeBondType:
-			GetLine2DCoords (1, &x1, &y1, &x2, &y2);
-			x = sqrt (square (x2 - x1) + square (y2 - y1));
-			dx = (y1 - y2) / x * pTheme->GetStereoBondWidth () / 2;
-			dy = (x2 - x1) / x * pTheme->GetStereoBondWidth () / 2;
-			gnome_canvas_path_def_moveto (path, x1 * pTheme->GetZoomFactor () + dx, y1 * pTheme->GetZoomFactor () + dy);
-			gnome_canvas_path_def_lineto (path, x2 * pTheme->GetZoomFactor () + dx, y2 * pTheme->GetZoomFactor () + dy);
-			gnome_canvas_path_def_lineto (path, x2 * pTheme->GetZoomFactor () -dx, y2 * pTheme->GetZoomFactor () -dy);
-			gnome_canvas_path_def_lineto (path, x1 * pTheme->GetZoomFactor () -dx, y1 * pTheme->GetZoomFactor () -dy);
-			gnome_canvas_path_def_closepath (path);
-			break;
-		case UndeterminedBondType:
-		{
-			GetLine2DCoords (1, &x1, &y1, &x2, &y2);
-			x1 *= pTheme->GetZoomFactor ();
-			y1 *= pTheme->GetZoomFactor ();
-			x2 *= pTheme->GetZoomFactor ();
-			y2 *= pTheme->GetZoomFactor ();
-			gnome_canvas_path_def_moveto (path, x1, y1);
-			double length;
-			length = sqrt (square (x2 - x1) + square (y2 - y1));
-			n = (int) length / 3;
-			int	s = 1;
-			dx = (x2 - x1) / n;
-			dy = (y2 - y1) / n;
-			x = x1;
-			y = y1;
-			for (int i = 1; i < n; i++) {
-				x1 = x + dx / 3 + dy /1.5 * s;
-				y1 = y + dy / 3 - dx /1.5 * s;
-				dx1 = x + dx / 1.5 + dy /1.5 * s;
-				dy1 = y + dy / 1.5 - dx /1.5 * s;
-				x += dx;
-				y += dy;
-				s *= -1;
-				gnome_canvas_path_def_curveto (path, x1, y1, dx1, dy1, x, y);
-			}
-			x1 = x + dx / 3 + dy /1.5 * s;
-			y1 = y + dy / 3 - dx /1.5 * s;
-			dx1 = x + dx / 1.5 + dy /1.5 * s;
-			dy1 = y + dy / 1.5 - dx /1.5 * s;
-			gnome_canvas_path_def_curveto (path, x1, y1, dx1, dy1, x2, y2);
-			break;
-		}
-	}
-	return path;
-}
-
-GnomeCanvasPathDef *Bond::BuildCrossingPathDef (WidgetData* pData)
-{
-	Theme *pTheme = pData->m_View->GetDoc ()->GetTheme ();
-	GnomeCanvasPathDef* path = NULL;
-	double x1, y1, x2, y2, x, dx, dy;
-	int i;
-	switch (GetType ()) {
-	case NormalBondType:
-		i = 1;
-		path = gnome_canvas_path_def_new ();
-		while (GetLine2DCoords (i++, &x1, &y1, &x2, &y2)) {
-			dx = (x2 - x1) / 10.;
-			dy = (y2 - y1) / 10.;
-			x1 += dx;
-			x2 -= dx;
-			y1 += dy;
-			y2 -= dy;
-			gnome_canvas_path_def_moveto (path, x1 * pTheme->GetZoomFactor (), y1 * pTheme->GetZoomFactor ());
-			gnome_canvas_path_def_lineto (path, x2 * pTheme->GetZoomFactor (), y2 * pTheme->GetZoomFactor ());
-		}
-		break;
-	case UpBondType:
-		break;
-	case DownBondType:
-		break;
-	case ForeBondType:
-		path = gnome_canvas_path_def_new ();
-		GetLine2DCoords (1, &x1, &y1, &x2, &y2);
-		dx = (x2 - x1) / 10.;
-		dy = (y2 - y1) / 10.;
-		x1 += dx;
-		x2 -= dx;
-		y1 += dy;
-		y2 -= dy;
-		x = sqrt(square(x2 - x1) + square(y2 - y1));
-		dx = (y1 - y2) / x * pTheme->GetStereoBondWidth () / 2;
-		dy = (x2 - x1) / x * pTheme->GetStereoBondWidth () / 2;
-		dx += (dx > 0.)? 1.: -1.;
-		dy += (dy > 0.)? 1.: -1.;
-		gnome_canvas_path_def_moveto (path, x1 * pTheme->GetZoomFactor () + dx, y1 * pTheme->GetZoomFactor () + dy);
-		gnome_canvas_path_def_lineto (path, x2 * pTheme->GetZoomFactor () + dx, y2 * pTheme->GetZoomFactor () + dy);
-		gnome_canvas_path_def_lineto (path, x2 * pTheme->GetZoomFactor () -dx, y2 * pTheme->GetZoomFactor () -dy);
-		gnome_canvas_path_def_lineto (path, x1 * pTheme->GetZoomFactor () -dx, y1 * pTheme->GetZoomFactor () -dy);
-		gnome_canvas_path_def_closepath (path);
-		break;
-	case UndeterminedBondType:
-		break;
-	}
-	return path;
-}*/
-
-/*void Bond::Update(GtkWidget* w) const
-{
-	if (!w || !m_order)
-		return;
-	WidgetData* pData = (WidgetData*) g_object_get_data(G_OBJECT(w), "data");
-	if (pData->Items[this] == NULL)
-		return;
-	Theme *pTheme = pData->m_View->GetDoc ()->GetTheme ();
-	bool result = false;
-	if (m_Crossing.size () > 0) {
-		map<Bond*, BondCrossing>::const_iterator i, iend = m_Crossing.end ();
-		for (i = m_Crossing.begin (); i != iend; i++)
-			if ((result |= (*i).second.is_before))
-				break;
-	}
-	GnomeCanvasPathDef* path;
-	void * obj;
-	GnomeCanvasGroup *group = pData->Items[this];
-	if (!group) {
-		Add (w);
-		return;
-	}
-	obj = g_object_get_data(G_OBJECT(group), "back");
-	if (result) {
-		path = const_cast <Bond *> (this)->BuildCrossingPathDef (pData);
-		if (path) {
-			if (obj)
-				g_object_set (obj, "bpath", path, NULL);
-			else {
-				GnomeCanvasItem *item = (m_type == NormalBondType || m_type == UndeterminedBondType)?
-					gnome_canvas_item_new (
-						group,
-						gnome_canvas_bpath_ext_get_type (),
-						"bpath", path,
-						"outline_color", "white",
-						"width_units", pTheme->GetBondWidth () * 3.,
-						NULL):
-					gnome_canvas_item_new (
-						group,
-						gnome_canvas_bpath_ext_get_type(),
-						"bpath", path,
-						"fill_color", "white",
-						"width_units", 0.,
-						NULL);
-				g_object_set_data (G_OBJECT (group), "back", item);
-				g_object_set_data (G_OBJECT (item), "object", (void *) this);
-				g_signal_connect (G_OBJECT (item), "event", G_CALLBACK (on_event), w);*/
-				/* now bring to front, FIXME: not secure ! */
-/*				gnome_canvas_item_lower_to_bottom (item);
-				gnome_canvas_item_raise_to_top (GNOME_CANVAS_ITEM (group));
-				gcu::Atom *pAtom = GetAtom (0);
-				if (pAtom->GetZ () != 6 || static_cast <gcp::Atom*> (pAtom)->GetShowSymbol ())
-					gnome_canvas_item_raise_to_top (GNOME_CANVAS_ITEM (pData->Items[pAtom]));
-				pAtom = GetAtom (1);
-				if (pAtom->GetZ () != 6 || static_cast <gcp::Atom*> (pAtom)->GetShowSymbol ())
-					gnome_canvas_item_raise_to_top (GNOME_CANVAS_ITEM (pData->Items[pAtom]));
-			}
-			gnome_canvas_path_def_unref (path);
-		}
-	} else if (obj) {
-		g_object_set_data (G_OBJECT(group), "back", NULL);
-	}
-	path = const_cast <Bond *> (this)->BuildPathDef (pData);
-	obj = g_object_get_data(G_OBJECT(group), "path");
-	g_object_set (obj, "bpath", path, NULL);
-	if (m_type == NormalBondType || m_type == UndeterminedBondType)
-		g_object_set (obj, "width_units", pTheme->GetBondWidth (), NULL);
-	gnome_canvas_path_def_unref (path);
-}*/
 	
 double Bond::GetYAlign ()
 {
@@ -1145,7 +798,7 @@ bool Bond::BuildContextualMenu (GtkUIManager *UIManager, Object *object, double
 
 void Bond::MoveToBack ()
 {
-	View *pView = reinterpret_cast<Document*> (GetDocument ())->GetView ();
+	View *pView = static_cast <Document *> (GetDocument ())->GetView ();
 	map<Bond*, BondCrossing>::iterator i, iend = m_Crossing.end ();
 	for (i = m_Crossing.begin (); i != iend; i++) {
 		if (m_level > (*i).first->m_level && m_type == (*i).first->m_type) {
@@ -1160,7 +813,7 @@ void Bond::MoveToBack ()
 
 void Bond::BringToFront ()
 {
-	View *pView = reinterpret_cast<Document*> (GetDocument ())->GetView ();
+	View *pView = static_cast <Document *> (GetDocument ())->GetView ();
 	map<Bond*, BondCrossing>::iterator i, iend = m_Crossing.end ();
 	for (i = m_Crossing.begin (); i != iend; i++) {
 		if (m_level < (*i).first->m_level && m_type == (*i).first->m_type) {
@@ -1239,4 +892,59 @@ string Bond::GetProperty (unsigned property) const
 	}
 }
 
+void Bond::AdjustPosition (double &x, double &y)
+{
+	if (m_order == 1 && m_type == NormalBondType)
+		return;
+	// evaluate where we are relative to the bond
+	int i = 1;
+	double x0, y0, x1, y1, x2, y2, x3, y3, d, l;
+	GetLine2DCoords (i++, &x1, &y1, &x2, &y2);
+	// we need the vector orthogonal to the bond and ending at x,y
+	x2 -= x1;
+	y2 -= y1;
+	// normalize
+	d = hypot (x2, y2);
+	x3 = x2 / d;
+	y3 = y2 / d;
+	l = (x - x1) * x3 + (y - y1) * y3;
+	x2 = x1 + l * x3;
+	y2 = y1 + l * y3;
+	x0 = x - x2;
+	y0 = y - y2;
+	switch (m_type) {
+	case NormalBondType:
+		while (i <= m_order) {
+			GetLine2DCoords (i++, &x1, &y1, &x2, &y2);
+			l = (x - x1) * x3 + (y - y1) * y3;
+			x2 = x - x1 - l * x3;
+			y2 = y - y1 - l * y3;
+			x1 = hypot (x2, y2);
+			y1 = hypot (x0, y0);
+			if ((x2 * x0 + y2 * y0 < 0 && x1 >= y1) || x1 < y1) {
+				x -= x2 - x0;
+				y -= y2 - y0;
+			}
+		}
+		return;
+	case UpBondType:
+		l = l / d;
+		break;
+	case DownBondType:
+		l = InvertWedgeHashes? l / d: 1. - l / d;
+		break;
+	case ForeBondType:
+	case UndeterminedBondType:
+		l = 1.;
+		break;
+	default:
+		return; // should not happen
+	}
+	Document *doc = static_cast <Document*> (GetDocument ());
+	Theme *theme = doc->GetTheme ();
+	d = hypot (x - x2, y - y2) / l / (theme->GetStereoBondWidth () -theme->GetBondWidth ()) * 2. * theme->GetZoomFactor ();
+	x += (x - x2) / d;
+	y += (y - y2) / d;
+}
+
 }	//	namespace gcp
